#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Collections.Generic;
using System.Threading;

namespace HeuristicLab.Parameters {
  /// <summary>
  /// A base class for parameters.
  /// </summary>
  [Item("Parameter", "A base class for parameters.")]
  [StorableClass]
  public abstract class Parameter : NamedItem, IParameter {
    public override Image ItemImage {
      get {
        if ((dataType != null) && (typeof(IOperator).IsAssignableFrom(dataType)))
          return HeuristicLab.Common.Resources.VS2008ImageLibrary.Method;
        else
          return base.ItemImage;
      }
    }
    public override bool CanChangeName {
      get { return false; }
    }
    public override bool CanChangeDescription {
      get { return false; }
    }

    [Storable]
    private Type dataType;
    public Type DataType {
      get { return dataType; }
    }
    protected Dictionary<int, IItem> cachedActualValues;
    public IItem ActualValue {
      get {
        int id = Thread.CurrentThread.ManagedThreadId;
        IItem value = null;
        lock (cachedActualValues) {
          cachedActualValues.TryGetValue(id, out value);
        }
        if (value == null) {
          value = GetActualValue();
          lock (cachedActualValues) {
            if (cachedActualValues.ContainsKey(id)) cachedActualValues[id] = value;
            else cachedActualValues.Add(id, value);
          }
        }
        return value;
      }
      set {
        int id = Thread.CurrentThread.ManagedThreadId;
        lock (cachedActualValues) {
          if (cachedActualValues.ContainsKey(id)) cachedActualValues[id] = value;
          else cachedActualValues.Add(id, value);
        }
        SetActualValue(value);
      }
    }
    private Dictionary<int, IExecutionContext> executionContexts;
    public IExecutionContext ExecutionContext {
      get {
        int id = Thread.CurrentThread.ManagedThreadId;
        IExecutionContext context = null;
        lock (executionContexts) {
          executionContexts.TryGetValue(id, out context);
        }
        return context;
      }
      set {
        IExecutionContext context = null;
        int id = Thread.CurrentThread.ManagedThreadId;
        lock (executionContexts) {
          executionContexts.TryGetValue(id, out context);
          if (value != context) {
            if (context == null) executionContexts.Add(id, value);
            else if (value == null) {
              executionContexts.Remove(id);
              lock (cachedActualValues) cachedActualValues.Remove(id);
            } else {
              executionContexts[id] = value;
              lock (cachedActualValues) cachedActualValues.Remove(id);
            }
          }
        }
      }
    }

    [StorableConstructor]
    protected Parameter(bool deserializing) : base(deserializing) {
      cachedActualValues = new Dictionary<int, IItem>();
      executionContexts = new Dictionary<int, IExecutionContext>();
    }
    protected Parameter(Parameter original, Cloner cloner)
      : base(original, cloner) {
      dataType = original.dataType;
      cachedActualValues = new Dictionary<int, IItem>();
      executionContexts = new Dictionary<int, IExecutionContext>();
      foreach (var item in original.executionContexts) {
        executionContexts.Add(item.Key, cloner.Clone(item.Value));
      }
    }
    protected Parameter()
      : base("Anonymous") {
      dataType = typeof(IItem);
      cachedActualValues = new Dictionary<int, IItem>();
      executionContexts = new Dictionary<int, IExecutionContext>();
    }
    protected Parameter(string name, Type dataType)
      : base(name) {
      if (dataType == null) throw new ArgumentNullException();
      this.dataType = dataType;
      cachedActualValues = new Dictionary<int, IItem>();
      executionContexts = new Dictionary<int, IExecutionContext>();
    }
    protected Parameter(string name, string description, Type dataType)
      : base(name, description) {
      if (dataType == null) throw new ArgumentNullException();
      this.dataType = dataType;
      cachedActualValues = new Dictionary<int, IItem>();
      executionContexts = new Dictionary<int, IExecutionContext>();
    }

    public override string ToString() {
      return Name;
    }

    protected abstract IItem GetActualValue();
    protected abstract void SetActualValue(IItem value);
  }
}
